--[[

This script is used to encode from sensor input (in mobicam2/mobicam3)
and component/hdmi input (from EVP) in different resolutions and
stream using UDP or RTP.
Also supports JPEG snapshots and MJPEG streaming.

The script detects the board and set parameters accordingly.
For EVP, following parameters are used as default:

LED = 0;
VIN = "component";
VIN_FORMAT = "720p";
--]]


----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

if board.getType() == "evp" then
  LED = 0;
  VIN = VIN or "component";       --can also use hdmi if there is no HDCP
  VIN_FORMAT = "720p";
end

LED = LED or 1;

if (LED == 1) then
  -- Turn off LED D1 to show that the system is initializing
  os.execute( "echo 1 > /proc/driver/gpio/native/64" )
end

SHOWRATE = SHOWRATE or 1;



-- Following ENABLE parameters controls encoding of various streams
--   1 : enable encoding
--   0 : do not enable encoding
-- The parameters with _LOCALSTORAGE suffix indicate separate instances
-- to be created for encoding to local media while those without this
-- suffix are for streaming. This will be needed if the stored stream
-- has different encoding characteristics than the streamed one 
-- Note: MJPEG is an exception where we do not create separate instance
ENABLE_HD = ENABLE_HD or 1;
ENABLE_HD_LOCALSTORAGE = ENABLE_HD_LOCALSTORAGE or 0;
ENABLE_D1 = ENABLE_D1 or 1;
ENABLE_D1_LOCALSTORAGE = ENABLE_D1_LOCALSTORAGE or 0;
ENABLE_CIF = ENABLE_CIF or 1;
ENABLE_CIF_LOCALSTORAGE = ENABLE_CIF_LOCALSTORAGE or 0;
ENABLE_JPEGSNAPSHOT = ENABLE_JPEGSNAPSHOT or 1;
ENABLE_MJPEG = ENABLE_MJPEG or 0;
ENABLE_MJPEG_LOCALSTORAGE = ENABLE_MJPEG_LOCALSTORAGE or 0;

-- Following enables object tracking
ENABLE_OT = ENABLE_OT or 0;

PROFILE_HD  = PROFILE_HD  or "high";
LEVEL_HD    = LEVEL_HD    or 40;
PROFILE_D1  = PROFILE_D1  or "high";    -- "baseline"
LEVEL_D1    = LEVEL_D1    or 40;        -- 31
PROFILE_CIF = PROFILE_CIF or "high";    -- "baseline"
LEVEL_CIF   = LEVEL_CIF   or 40;        -- 31

-- Following parameters control if we need to store whatever is streamed
-- in local media. Enabling these will not create another instance of 
-- the encoder  like the way done in the above parameters with suffix
-- _LOCALSTORAGE. These parameters will only be relevant when the 
-- corresponding ENABLE parameter is set.
STORE_HD = STORE_HD or 0;
STORE_D1 = STORE_D1 or 0;
STORE_CIF = STORE_CIF or 0;
STORE_MJPEG = STORE_MJPEG or 0;

-- File format of files saved on local media
--   "h4v"
--   "file"         (QBOX format)
OUTFILE_FORMAT  = OUTFILE_FORMAT or "h4v";

-- Following name is used for local storage. Note that it is assumed
-- that we either store what is streamed or encoded from the separate 
-- instance created as described above but do not enable both for
-- the same resolution at the same time
if (OUTFILE_FORMAT  == "h4v") then
  OUTFILENAME_HD = OUTFILENAME_HD or "/media/mmcblk0p1/hd.h264"
  OUTFILENAME_D1 = OUTFILENAME_D1 or "/media/mmcblk0p1/d1.h264"
  OUTFILENAME_CIF = OUTFILENAME_CIF or "/media/mmcblk0p1/cif.h264"
else
  OUTFILENAME_HD = OUTFILENAME_HD or "/media/mmcblk0p1/hd.qbx"
  OUTFILENAME_D1 = OUTFILENAME_D1 or "/media/mmcblk0p1/d1.qbx"
  OUTFILENAME_CIF = OUTFILENAME_CIF or "/media/mmcblk0p1/cif.qbx"
end

JPEGSNAPSHOT_BASEFILENAME = JPEGSNAPSHOT_BASEFILENAME or "/media/mmcblk0p1/snap"
OUTFILENAME_MJPEG = OUTFILENAME_MJPEG or "/media/mmcblk0p1/test.mjpeg"

-- HD stream video bitrate
VBITRATE_HD = VBITRATE_HD or 6000000;

if (ENABLE_D1 == 1)  or (ENABLE_D1_LOCALSTORAGE == 1) then
  -- 2nd D1 stream
  ENCW_D1=720;
  ENCH_D1=480;
  if ENABLE_D1 == 1 then
    -- Following parameter relevant for UDP only
    PORT_D1 = 9750;
    VBITRATE_D1 = VBITRATE_D1 or 2000000;
  end
end

if (ENABLE_CIF == 1) or (ENABLE_CIF_LOCALSTORAGE == 1) then
  -- 3nd CIF stream
  ENCW_CIF=360;
  ENCH_CIF=240;
  if ENABLE_CIF == 1 then
    -- Following parameter relevant for UDP only
    PORT_CIF = 9700;
    VBITRATE_CIF = VBITRATE_CIF or 500000;
  end
end

VBITRATE_HD_LOCALSTORAGE = VBITRATE_HD_LOCALSTORAGE or 6000000;
VBITRATE_D1_LOCALSTORAGE = VBITRATE_D1_LOCALSTORAGE or 2000000;
VBITRATE_CIF_LOCALSTORAGE = VBITRATE_CIF_LOCALSTORAGE or 500000;

-- Controls the GOP structure for local storage
-- "IBBRBP" : Pyramidal GOP structure
-- "IP" : No B frames
GOP_STRUCTURE_HD_LOCALSTORAGE =  GOP_STRUCTURE_HD_LOCALSTORAGE or "IBBRBP";
GOP_STRUCTURE_D1_LOCALSTORAGE =  GOP_STRUCTURE_D1_LOCALSTORAGE or "IBBRBP";
GOP_STRUCTURE_CIF_LOCALSTORAGE =  GOP_STRUCTURE_CIF_LOCALSTORAGE or "IBBRBP";

-- send this many blocks per interrupt
HD_AVCENC_XFER_SIZE      = HD_AVCENC_XFER_SIZE    or 1
D1_AVCENC_XFER_SIZE      = D1_AVCENC_XFER_SIZE    or 1
CIF_AVCENC_XFER_SIZE     = CIF_AVCENC_XFER_SIZE   or 1

-- Interval between interrupts (if non-zero) in microseconds
INTERRUPT_PERIOD                = INTERRUPT_PERIOD              or 0;

-- For MJPEG, use a single encoder instance for either streaming or local storage
if ENABLE_MJPEG == 1 or ENABLE_MJPEG_LOCALSTORAGE == 1 then
  PORT_MJPEG = 9900;     -- used only with STREAMING=1
-- If the width and height are kept same as LVPP_OUTPUT_WIDTH  and LVPP_OUTPUT_HEIGHT
-- respectively, then no separate NVPP instance will be created
  ENCW_MJPEG=720;
  ENCH_MJPEG=480;
  -- To save the encoded bitstream or not
  BITSTREAM_BUFFER = BITSTREAM_BUFFER or 1;

  BITRATE_MJPEG = BITRATE_MJPEG or 6000000
  -- Quality factor, 0 (worst) - 100 (best)
  QFACTOR         = QFACTOR or 10;

  -- Encoding interval in 90khz ticks
  -- -1 means encoding every incoming frames
  ENC_INTERVAL    = ENC_INTERVAL or -1;

  -- Encoding start index. Use it to skip beginning incoming video frames
  ENC_START_INDEX = ENC_START_INDEX or 0

  -- Max number of frames to be encoded
  -- 0 means don't encode at all; encoder sit idle
  -- -1 means to encode as much as possible
  ENC_NUM         = ENC_NUM or -1
-- The following parameters are not used in SDK 4.01 or later versions
-- See the supported mjpegenc.lua script in SDK 4.0 for details on these
-- parameters 
  PERIOD          = PERIOD or 120;
  RES             = 0;
  TOLERANCE       = TOLERANCE or 2;
-- Following parameter is used in SDK 4.01 or later versions
-- See the supported mjpegenc.lua script in SDK 4.01 for details on this
-- parameter 
  BITRATE_BEHAVIOR = BITRATE_BEHAVIOR or 0;
end

if (ENABLE_JPEGSNAPSHOT == 1) then
  -- If the width and height are same as the lvpp output width and height 
  -- respectively, then no NVPP instance will be created which will save memory
  -- Specify 0 to use lvpp output width and height; later on it will change to
  -- lvpp width and height respectively if 0 
  ENCW_JPEGSNAPSHOT=0;
  ENCH_JPEGSNAPSHOT=0;
  -- Use smaller buffer when doing JPEG encode at lower resolutions to save
  -- memory.

  --[[
  It is a  requirement that the output buffer size must always be at least twice the size of the 
  guardband.
  For a 1600x1200 image, the guard band size can be calculated as follows:
  The macro block size is 16x16 and guardband per macro block is 300 bytes. So the total guard band
  size is 300x((1600x1200)/(16x16)) = 2250000  which is 2250000/(1024x1024) = 2.15MB.
  So buffer size should be at least 2.15*2 = 4.3MB
  --]]

  JPEGSNAPSHOT_ENC_BUFFER_SIZE = 4.3*1024*1024;
end

-- Controls live vs reconstructed preview
-- 0 : no preview
-- 1 : live
-- 2 : reconstructed
PREVIEW =  PREVIEW or 1;

-- Allow video only encode
VIDEO_ONLY = VIDEO_ONLY or 0

-- Audio bitrate
ABITRATE =  ABITRATE or 64000;

-- Audio sample rate
SAMPLERATE =  SAMPLERATE or 48000;

-- Controls the audio encoder type
--   "AAC-LC"
--   "G711-A-LAW"
--   "G711-MU-LAW"
--   "G711" (defaults to mu-law)
--   "G726"
AUDIO_ENCODER = AUDIO_ENCODER or "AAC-LC";

-- Audio frame size (may be overridden depending on codec)
AUDIO_FRAME_SIZE = AUDIO_FRAME_SIZE or 1024;

VIN		= VIN or "sensor";
VIN_FORMAT	= VIN_FORMAT or "720p30"

VOUT	= VOUT or "composite";
VOUT_FORMAT	= VOUT_FORMAT or "480i"

AUDIOSTREAMID = 1;
videoStreamId_hd = 2;
videoStreamId_d1 = 0x12;
videoStreamId_cif = 0x22;
videoStreamId_hd_localstorage = 0x32;
videoStreamId_d1_localstorage = 0x42;
videoStreamId_cif_localstorage = 0x52;
jpegStreamId = 3;
mjpegStreamId = 4;

-- Controls network streaming configuration
-- 0: no streaming
-- 1: udp (elementary stream)
-- 2: rtp
STREAMING       = STREAMING or 2;
-- Following parameters relevant for UDP only
-- Set to 0 for streaming elementary H264 to PC client (like VLC)
-- Set to 1 for streaming QBOX to another board
BOARD_TO_BOARD = BOARD_TO_BOARD or 0;
HOST            = HOST or "192.168.1.110";
PORT_HD         = PORT_HD or 9800;

-- RTP/RTSP Settings
-- ip address of rtp server.
SERVER_IP = SERVER_IP or "NULL"; -- NULL means find automatically
SERVER_PORT = SERVER_PORT or 554;
-- session identifier for rtp server
SESSION_ID = SESSION_ID or "test_HD";
--username for session
USER_NAME = USER_NAME or "NULL";
--password for the session
PASSWORD = PASSWORD or "NULL";
-- Multicast IP, make it NULL for unicast
MULTICAST_IP = MULTICAST_IP or "NULL";
-- Multicast Video Port
MULTICAST_VID_PORT = MULTICAST_VID_PORT or 4567;
-- Multicast Audio Port
MULTICAST_AUD_PORT = MULTICAST_AUD_PORT or 1234;

SESSION_ID1 = SESSION_ID1 or "test_D1";
--username for session
USER_NAME1 = USER_NAME1 or "NULL";
--password for the session
PASSWORD1 = PASSWORD1 or "NULL";
-- Multicast IP, make it NULL for unicast
MULTICAST_IP1 = MULTICAST_IP1 or "NULL";
-- Multicast Video Port
MULTICAST_VID_PORT1 = MULTICAST_VID_PORT1 or 5678;
-- Multicast Audio Port
MULTICAST_AUD_PORT1 = MULTICAST_AUD_PORT1 or 2345;

SESSION_ID2 = SESSION_ID2 or "test_CIF";
--username for session
USER_NAME2 = USER_NAME2 or "NULL";
--password for the session
PASSWORD2 = PASSWORD2 or "NULL";
-- Multicast IP, make it NULL for unicast
MULTICAST_IP2 = MULTICAST_IP2 or "NULL";
-- Multicast Video Port
MULTICAST_VID_PORT2 = MULTICAST_VID_PORT2 or 6789;
-- Multicast Audio Port
MULTICAST_AUD_PORT2 = MULTICAST_AUD_PORT2 or 3456;

SESSION_ID3 = SESSION_ID3 or "test_MJPEG";
--username for session
USER_NAME3 = USER_NAME3 or "NULL";
--password for the session
PASSWORD3 = PASSWORD3 or "NULL";
-- Multicast IP, make it NULL for unicast
MULTICAST_IP3 = MULTICAST_IP3 or "NULL";
-- Multicast Video Port
MULTICAST_VID_PORT3 = MULTICAST_VID_PORT3 or 4444;

-- rtp server mode
SERVER_MODE = SERVER_MODE or 0;
if (VIDEO_ONLY == 0) then
  if (AUDIO_ENCODER == "AAC-LC") then
    RTP_AUDIO="aac";
  elseif (AUDIO_ENCODER == "G711-A-LAW") then
    RTP_AUDIO="g711a";
  elseif (AUDIO_ENCODER == "G711-MU-LAW")  or (AUDIO_ENCODER == "G711") then
    RTP_AUDIO="g711u";
  elseif (AUDIO_ENCODER == "G726") then
    RTP_AUDIO="g726";
  end 
end
-- MTU size for UDP in RTP streaming
UDP_MTU_SIZE = UDP_MTU_SIZE or 1472;
-- MTU size for TCP in RTP streaming
TCP_MTU_SIZE = TCP_MTU_SIZE or 8192;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------



-------------------------------------

ENC_GUARD_BAND_PER_MB = 400;

BITSTREAM_OUTPUT = 1;

if (VIN_FORMAT == "720p25") then
  VIN_STC = 3600;
  VIN_OUTPUT_STC = 3600;
  HSIZE         = 1280;
  VSIZE         = 720;
  FPS           = 25;
  ENC_OUTPUT_BUFFER  = 4*1024*1024;
  NUM_VIN_FRAMES = 10;
elseif (VIN_FORMAT == "720p30") then
  VIN_STC = 3000;
  VIN_OUTPUT_STC = 3000;
  HSIZE         = 1280;
  VSIZE         = 720;
  FPS           = 30;
  ENC_OUTPUT_BUFFER  = 4*1024*1024;
  NUM_VIN_FRAMES = 10;
elseif (VIN_FORMAT == "720p") then
  -- We will use input decimation so keep VIN_STC for 30frames/sec
  VIN_STC = 3000;
  VIN_OUTPUT_STC = 3000;
  HSIZE         = 1280;
  VSIZE         = 720;
  FPS           = 30;
  ENC_OUTPUT_BUFFER  = 4*1024*1024;
  NUM_VIN_FRAMES = 10;
elseif (VIN_FORMAT == "uxgap15") then
  VIN_STC = 6000;
  VIN_OUTPUT_STC = 6000;
  HSIZE         = 1600;
  VSIZE         = 1200;
  FPS           = 15;
  ENC_OUTPUT_BUFFER  = 6*1024*1024;
  NUM_VIN_FRAMES = 8;     -- Have to reduce to avoid running out of memory
end

if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
  LVPP_OUTPUT_WIDTH  = HSIZE;
  LVPP_OUTPUT_HEIGHT = VSIZE;
elseif (ENABLE_D1 == 1) or (ENABLE_D1_LOCALSTORAGE == 1) then
  LVPP_OUTPUT_WIDTH  = ENCW_D1;
  LVPP_OUTPUT_HEIGHT = ENCH_D1;
else
-- CIF only (not a practical use case)
  LVPP_OUTPUT_WIDTH  = HSIZE;
  LVPP_OUTPUT_HEIGHT = VSIZE;
end

if (ENABLE_JPEGSNAPSHOT == 1) then
  if (ENCW_JPEGSNAPSHOT == 0) then 
    ENCW_JPEGSNAPSHOT = LVPP_OUTPUT_WIDTH;
  end
  if (ENCH_JPEGSNAPSHOT == 0) then 
    ENCH_JPEGSNAPSHOT = LVPP_OUTPUT_HEIGHT;
  end
end
 
if (GOP_STRUCTURE_HD_LOCALSTORAGE == "IP") then
  gopstruct_hd_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IP;
elseif  (GOP_STRUCTURE_HD_LOCALSTORAGE == "IBBRBP") then
  gopstruct_hd_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
else
  print("Invalid GOP structure " .. GOP_STRUCTURE_HD_LOCALSTORAGE .. "\n");
end

if (GOP_STRUCTURE_D1_LOCALSTORAGE == "IP") then
  gopstruct_d1_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IP;
elseif  (GOP_STRUCTURE_D1_LOCALSTORAGE == "IBBRBP") then
  gopstruct_d1_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
else
  print("Invalid GOP structure " .. GOP_STRUCTURE_D1_LOCALSTORAGE .. "\n");
end

if (GOP_STRUCTURE_CIF_LOCALSTORAGE == "IP") then
  gopstruct_cif_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IP;
elseif  (GOP_STRUCTURE_CIF_LOCALSTORAGE == "IBBRBP") then
  gopstruct_cif_localstorage = Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP;
else
  print("Invalid GOP structure " .. GOP_STRUCTURE_CIF_LOCALSTORAGE .. "\n");
end


-- audio parameters for audio
if (VIDEO_ONLY == 0) then
   -- setup audio parameters with over-rides as necessary
   QAC_AOT = 2;
   Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
   if (AUDIO_ENCODER == "AAC-LC") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_QAC;
      AUDIO_FRAME_SIZE = 1024;
      QAC_AOT = 2;
   elseif (AUDIO_ENCODER == "G711-A-LAW") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_A_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G711-MU-LAW") or (AUDIO_ENCODER == "G711") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q711;
      Q711_LAW = Q_AUDENC_CFP_Q711_U_LAW;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      ABITRATE = 64000;
   elseif (AUDIO_ENCODER == "G726") then
      AUDIO_CODEC = Q_AUDENC_CFP_CODEC_Q726;
      -- AUDIO_FRAME_SIZE is flexible
      SAMPLERATE = 8000;
      AUDIO_STEREO = 0;
      if (ABITRATE ~= 40000) and (ABITRATE ~= 32000) and (ABITRATE ~= 24000) and (ABITRATE ~= 16000) then
         ABITRATE = 40000; -- set usable default
      end
   else
      error("Invalid audio encoder " .. tostring(AUDIO_ENCODER))
   end
end

if (ENABLE_OT > 0) then
  -- OSD_BUFFER_WIDTH is the physical width of the buffer in pixels which is
  -- typically 2048/bytes per pixel if 16bpp
  OSD_BUFFER_WIDTH        = OSD_BUFFER_WIDTH or 2048;              -- 2048
  OSD_BUFFER_HEIGHT       = OSD_BUFFER_HEIGHT or 480;
  OSD_DISPLAY_WIDTH       = OSD_DISPLAY_WIDTH or 720;
  OSD_DISPLAY_HEIGHT      = OSD_DISPLAY_HEIGHT or 480;
  OSD_X_OFFSET            = OSD_X_OFFSET or 0;
  OSD_Y_OFFSET            = OSD_Y_OFFSET or 0;
  OSD_MEMORY_SIZE         = 4*1024*1024;
  ZOOM                    = 1;       --Must always be 1

end
----- function definitions ----------


if (ENABLE_HD == 1) then
  function recordhd()
    if (STREAMING == 1) then
      if (BOARD_TO_BOARD == 0) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=768", HOST, PORT_HD), sid = {videoStreamId_hd}});
      elseif (VIDEO_ONLY == 1) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_HD), sid = {videoStreamId_hd}});
      else
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_HD), sid = {videoStreamId_hd, AUDIOSTREAMID}});
      end
    elseif(STREAMING ==2) then
      if (VIDEO_ONLY == 0) then
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=%s,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,mpta=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID, RTP_AUDIO, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP, MULTICAST_VID_PORT, MULTICAST_AUD_PORT,USER_NAME,PASSWORD), sid = {videoStreamId_hd, AUDIOSTREAMID}});
      else
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=null,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP, MULTICAST_VID_PORT,USER_NAME,PASSWORD), sid = {videoStreamId_hd}});
      end
    end
    if (STORE_HD == 1) then
      if (OUTFILE_FORMAT == "h4v") then
        ws:open({type="h4v", file=OUTFILENAME_HD, sid = {videoStreamId_hd}});
      else
        ws:open({type="file", file=OUTFILENAME_HD, sid = {videoStreamId_hd, AUDIOSTREAMID}});
      end
    end

    avcenchd:bindVideoInput({src=lvpp0});
    avsn0:start();
    avcenchd:record();
    if (ENABLE_OT == 1) then
      -- The OSD window size is adjusted for aspect ratio
      if( VIN_FORMAT == "720p30" or VIN_FORMAT == "720p25" or VIN_FORMAT == "720p") then
        --Adjust OSD output for aspect ratio
        enableTracking({avcenc=avcenchd["handle"], xoff=0, yoff=60, width=720, height=360, width_org=HSIZE, height_org=VSIZE });
      else
        enableTracking({avcenc=avcenchd["handle"], xoff=0, yoff=0, width=720, height=480, width_org=HSIZE, height_org=VSIZE });
      end
      setupTracking({zoom=ZOOM, rect_thick=2, fill_contour=0, bitmap_contour=0});
    end

  end

  function stophd()
    avcenchd:unbindVideoInput();
    avcenchd:flush();
    avcenchd:waitForLastBlock();
    avcenchd:stop();
    avsn0:stop();
  end
end

if (ENABLE_D1 == 1) then
  function recordd1()
    if (STREAMING == 1) then
      if (BOARD_TO_BOARD == 0) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=768", HOST, PORT_D1), sid = {videoStreamId_d1}});
      elseif (VIDEO_ONLY == 1) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_D1), sid = {videoStreamId_d1}});
      else
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_D1), sid = {videoStreamId_d1, AUDIOSTREAMID}});
      end
    elseif(STREAMING ==2) then
      if (VIDEO_ONLY == 0) then
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=%s,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,mpta=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID1, RTP_AUDIO, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP1, MULTICAST_VID_PORT1, MULTICAST_AUD_PORT1,USER_NAME1,PASSWORD1), sid = {videoStreamId_d1, AUDIOSTREAMID}});
      else
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=null,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID1, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP1, MULTICAST_VID_PORT1,USER_NAME1,PASSWORD1), sid = {videoStreamId_d1}});
      end
    end
    if (STORE_D1 == 1) then
      if (OUTFILE_FORMAT == "h4v") then
        ws:open({type="h4v", file=OUTFILENAME_D1, sid = {videoStreamId_d1}});
      else
        ws:open({type="file", file=OUTFILENAME_D1, sid = {videoStreamId_d1, AUDIOSTREAMID}});
      end
    end

    if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
      nvppencd1:start();
    else
      avcencd1:bindVideoInput({src=lvpp0});
      avsn0:start();
    end
    avcencd1:record();
    if (ENABLE_OT == 1) then
      -- The OSD window size is adjusted for aspect ratio
      if(ENABLE_HD == 0) then
        enableTracking({avcenc=avcencd1["handle"], xoff=0, yoff=60, width=720, height=360, width_org=720, height_org=480 });
        setupTracking({zoom=ZOOM, rect_thick=1, fill_contour=0, bitmap_contour=0});
      end
    end
  end

  function stopd1()
    if (ENABLE_HD == 0) then
      avcencd1:unbindVideoInput();
    else
      nvppencd1:flush();
    end
    avcencd1:flush();
    avcencd1:waitForLastBlock();
    avcencd1:stop();
    if (ENABLE_HD == 0) then
      avsn0:stop();
    end
  end
end

if (ENABLE_CIF == 1) then
  function recordcif()
    if (STREAMING == 1) then
      if (BOARD_TO_BOARD == 0) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=768", HOST, PORT_CIF), sid = {videoStreamId_cif}});
      elseif (VIDEO_ONLY == 1) then
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_CIF), sid = {videoStreamId_cif}});
      else
        ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_CIF), sid = {videoStreamId_cif, AUDIOSTREAMID}});
      end
    elseif(STREAMING ==2) then
      if (VIDEO_ONLY == 0) then
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=%s,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,mpta=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID2, RTP_AUDIO, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP2, MULTICAST_VID_PORT2, MULTICAST_AUD_PORT2,USER_NAME2,PASSWORD2), sid = {videoStreamId_cif, AUDIOSTREAMID}});
      else
        ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,vid=h264,aud=null,mod=%d,ums=%d,tms=%d,mip=%s,mptv=%d,user=%s,pass=%s", SERVER_IP, SERVER_PORT, SESSION_ID2, SERVER_MODE, UDP_MTU_SIZE, TCP_MTU_SIZE, MULTICAST_IP2, MULTICAST_VID_PORT2,USER_NAME2,PASSWORD2), sid = {videoStreamId_cif}});
      end
    end
    if (STORE_CIF == 1) then
      if (OUTFILE_FORMAT == "h4v") then
        ws:open({type="h4v", file=OUTFILENAME_CIF, sid = {videoStreamId_cif}});
      else
        ws:open({type="file", file=OUTFILENAME_CIF, sid = {videoStreamId_cif, AUDIOSTREAMID}});
      end

    end

    avcenccif:record();
    nvppenccif:start();
  end

  function stopcif()
    nvppenccif:flush();
    avcenccif:flush();
    avcenccif:waitForLastBlock();
    avcenccif:stop();
  end
end 


-- MJPEG record/stop functions will only open/close the write streamer as MJPEG stop and unbindVideoInput are not implemented
if (ENABLE_MJPEG == 1) or (ENABLE_MJPEG_LOCALSTORAGE == 1) then
  function recordmjpeg()
    --records motion-jpeg:
    if (STREAMING == 1) then
      ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_MJPEG), sid = {mjpegStreamId}});
    elseif (STREAMING == 2) then
       ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,ums=%d,tms=%d,mip=%s,mptv=%d,user=%s,pass=%s",
             SERVER_IP, SERVER_PORT , SESSION_ID3 , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE, MULTICAST_IP3, MULTICAST_VID_PORT3,USER_NAME3,PASSWORD3), sid = {mjpegStreamId}});
    end
    if (ENABLE_MJPEG_LOCALSTORAGE == 1) or (STORE_MJPEG == 1) then
      outFileStr = "basefilename="..OUTFILENAME_MJPEG;
      outFileStr = outFileStr..",single=0";
	 
      ws:open({type="jpeg", file=outFileStr, sid={mjpegStreamId}});
    end
  end

  function stopmjpeg()
    if (STREAMING == 1) then
      ws:close({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=0,max=768", HOST, PORT_MJPEG), sid = {mjpegStreamId}});
    elseif (STREAMING == 2) then
       ws:close({type="rtp", file=string.format("sip=%s,prt=%d,ses=%s,mod=%d,vid=mjpeg,ums=%d,tms=%d,user=%s,pass=%s",
             SERVER_IP, SERVER_PORT , SESSION_ID3 , SERVER_MODE , UDP_MTU_SIZE , TCP_MTU_SIZE,USER_NAME3,PASSWORD3), sid = {mjpegStreamId}});
    end
    if (ENABLE_MJPEG_LOCALSTORAGE == 1) or (STORE_MJPEG == 1) then
      outFileStr = "basefilename="..OUTFILENAME_MJPEG;
      outFileStr = outFileStr..",single=0";

      ws:close({type="jpeg", file=outFileStr, sid={mjpegStreamId}});
    end
  end
end

function CreateSnapShotObj()
   if ENABLE_JPEGSNAPSHOT == 1 then
      if(ENCH_JPEGSNAPSHOT ~= LVPP_OUTPUT_HEIGHT) or (ENCW_JPEGSNAPSHOT ~= LVPP_OUTPUT_WIDTH) then
	 nvpp_jpegsnapshot = sysctl0:nvppCreate({name="nvppj", maxWidth=ENCW_JPEGSNAPSHOT, maxHeight=ENCH_JPEGSNAPSHOT,  numVidFrame=3, maxPMEDSFactor=0});
      end
      jpegenc0 = sysctl0:jpegencCreate({name="jpegenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE, outputbuffersize=JPEGSNAPSHOT_ENC_BUFFER_SIZE});

      if(ENCH_JPEGSNAPSHOT ~= LVPP_OUTPUT_HEIGHT) or (ENCW_JPEGSNAPSHOT ~= LVPP_OUTPUT_WIDTH) then
	 jpegenc0:bindVideoInput({src=nvpp_jpegsnapshot});
      else
	 jpegenc0:bindVideoInput({src=lvpp0});
      end
   end
end

function CreateMJPEGObj()
  if ENABLE_MJPEG == 1 or ENABLE_MJPEG_LOCALSTORAGE == 1 then
    if(ENCH_MJPEG ~= LVPP_OUTPUT_HEIGHT) or (ENCW_MJPEG ~= LVPP_OUTPUT_WIDTH) then
      nvpp_mjpeg = sysctl0:nvppCreate({name="nvppmj", maxWidth=ENCW_MJPEG, maxHeight=ENCH_MJPEG, maxPMEDSFactor=0});
    end
    jpegenc1 = sysctl0:jpegencCreate({name="jpegenc0", ws=ws, maxwidth=1280, maxheight=720, outputbuffersize=2621440});

    if(ENCH_MJPEG ~= LVPP_OUTPUT_HEIGHT) or (ENCW_MJPEG ~= LVPP_OUTPUT_WIDTH) then
      jpegenc1:bindVideoInput({src=nvpp_mjpeg});
    else
      jpegenc1:bindVideoInput({src=lvpp0});
    end
   end
end

function SetupSnapShotObj()
   if ENABLE_JPEGSNAPSHOT == 1 then
      --takes snap-shots:
      outFileStrSnapshot = "basefilename="..JPEGSNAPSHOT_BASEFILENAME;
      outFileStrSnapshot = outFileStrSnapshot..",single=1";
      
      --    outFileStrSnapshot="basefilename=/data/snapshot,single=1";
      ws:open({type="jpeg", file=outFileStrSnapshot, sid={jpegStreamId}});
      --setup encoding interval in 90khz ticks
      --    jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=54000000}); --10 minutes once
      --when does the JPEG encoder start capturing(encoding) incoming V frames?
      jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=0});
      --how many V frames to be captured(encoded)? (-1 means as many as possible)
      --For snapshot, we want to keep it 0
      jpegenc0:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=0});
      jpegenc0:activateCfg();
      
      jpegenc0:setAckInterval({interval=1});
      jpegenc0:setStreamID({streamid=jpegStreamId});
      jpegenc0:setQualityFactor({qualityfactor=55});
      
   end
end

function SetupMJPEGObj()
   if (ENABLE_MJPEG == 1) or (ENABLE_MJPEG_LOCALSTORAGE == 1) then
      recordmjpeg();
      jpegenc1:setParam({param=Q_JPEGENC_CMP_CAPTURE_INTERVAL, value=ENC_INTERVAL});
      jpegenc1:setParam({param=Q_JPEGENC_CMP_CAPTURE_START_INDEX, value=ENC_START_INDEX});
      jpegenc1:setParam({param=Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, value=ENC_NUM});
      jpegenc1:setParam({param=Q_JPEGENC_CMP_BITSTREAM_BUFFER, value=BITSTREAM_BUFFER});
      jpegenc1:activateCfg();
      jpegenc1:setAckInterval({interval=ACK_INTERVAL});
      jpegenc1:setStreamID({streamid=mjpegStreamId});
      jpegenc1:setQualityFactor({qualityfactor=QFACTOR});
      
      -- Code for SDK 4.0
      --    jpegenc1:bitrateControl({bitrate=BITRATE_MJPEG,fps=FPS,format=RES,period=PERIOD,tolerance=TOLERANCE});
      -- Code for SDK 4.01
      jpegenc1:bitrateControl({bitrate=BITRATE_MJPEG,fps=FPS,behavior=BITRATE_BEHAVIOR});
      
   end
end

function LinkSnapShotObj()
   if (ENABLE_JPEGSNAPSHOT == 1) then
      if ((ENCH_JPEGSNAPSHOT ~= LVPP_OUTPUT_HEIGHT) or (ENCW_JPEGSNAPSHOT ~= LVPP_OUTPUT_WIDTH)) then
	 --NVPP settings for JPEG snapshot
	 Setup_VPP(nvpp_jpegsnapshot, LVPP_OUTPUT_WIDTH, LVPP_OUTPUT_HEIGHT, 0, 1, 2, 2)
	 nvpp_jpegsnapshot:setParam({param=Q_LVPP_CMP_PMEDS_EN,           value=0});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_JPEGSNAPSHOT});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_JPEGSNAPSHOT});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=VIN_OUTPUT_STC});
	 nvpp_jpegsnapshot:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
	 nvpp_jpegsnapshot:activateCfg();
	 nvpp_jpegsnapshot:bindInput({src=lvpp0});
      end
   end
end

function LinkMJPEGObj()
  if (ENABLE_MJPEG == 1) or (ENABLE_MJPEG_LOCALSTORAGE == 1) then
    if(ENCH_MJPEG ~= LVPP_OUTPUT_HEIGHT) or (ENCW_MJPEG ~= LVPP_OUTPUT_WIDTH) then
      Setup_VPP(nvpp_mjpeg, LVPP_OUTPUT_WIDTH, LVPP_OUTPUT_HEIGHT, 0, 1, 2, 2)
      nvpp_mjpeg:setParam({param=Q_LVPP_CMP_PMEDS_EN,           value=0});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_MJPEG});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_MJPEG});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=VIN_OUTPUT_STC});
      nvpp_mjpeg:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
      nvpp_mjpeg:activateCfg();
      nvpp_mjpeg:bindInput({src=lvpp0});
    end
  end
end

function shot()
   jpegenc0:snapshot({shots=1});
end

function br(bitrate)
   avcenchd:setVideoEncRateControlParam({param=Q_AVCENC_RC_CMP_BITRATE, value=bitrate});
   avcenchd:activateVideoEncCfg();
end

if (ENABLE_D1 == 1) and (ENABLE_HD == 1) then

  function cropd1(crop_width , crop_height , horz_offset, vert_offset)
    if(crop_width > 0) and (crop_height > 0) then
      nvppencd1:crop({cropWidth=crop_width, cropHeight=crop_height, horzOffset=horz_offset, vertOffset=vert_offset});
      nvppencd1:activateCfg();
    end
  end

  function mycrop(width,height,intX,intY)
    cropd1(width, height, intX, intY);
  end

  function uncrop()
    cropd1(1280, 720, 0, 0);
  end

end
------------- end of functions -----------------------

sysctl0:setVerboseLevel({param=3});

-- Interrupt aggregation (available in SDK 4.01 and above)
if (INTERRUPT_PERIOD > 0) then
  print("Sysctl: Aggregating interrupts every "..INTERRUPT_PERIOD.." usecs\n");
  sysctl0:configure({param=Q_SYS_CFG_EVENT_CYCLE_IN_USEC, value=INTERRUPT_PERIOD});
end

if (VIN == "sensor") then
  -- Use external sync
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x381028, interleave=1});
else
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
end

sysctl0:setVPortMode({port=1, live=0});

-- Set up the video input port
board.setVideoInput({port=0, format=VIN_FORMAT, mode=string.format("%s",VIN)});

-- Set up video output port
board.setVideoOutput({port=0, format=VOUT_FORMAT, mode=VOUT});

if (VIDEO_ONLY == 0) then
	board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
end

-- Make uma 1 to reduce memory copies as we use MG3500 in SOC mode
-- which has a unified memory
ws = sysctl0:createWriteStreamer({name="ws0", uma=1, sendfile=1});
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  if(ENABLE_OT == 0) then
    avoc0=sysctl0:avocCreate({name="avoc0"});
  else
    avoc0=sysctl0:avocCreate({name="avoc0", gfx0Size=OSD_MEMORY_SIZE, allocGfx=1});
  end
end
lvpp0=sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=LVPP_OUTPUT_WIDTH, maxHeight=LVPP_OUTPUT_HEIGHT, numVidFrame=NUM_VIN_FRAMES});

avsn0=sysctl0:avsnCreate({name="avsn0"});
if(ENABLE_HD == 1) then
  avcenchd = sysctl0:avcencCreate({name="avcenchd", ws=ws, maxwidth=HSIZE, maxheight=VSIZE, outputbuffersize = ENC_OUTPUT_BUFFER});
end
if(ENABLE_HD_LOCALSTORAGE == 1) then
  avcenchdl = sysctl0:avcencCreate({name="avcenchdl", ws=ws, maxwidth=HSIZE, maxheight=VSIZE, outputbuffersize = ENC_OUTPUT_BUFFER});
end

if ENABLE_D1 == 1 or ENABLE_D1_LOCALSTORAGE == 1 then
  -- 2nd D1
  -- Create NVPP instance only if HD encoding is also needed
  if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
    nvppencd1 = sysctl0:nvppCreate({name="nvppd1", maxWidth=ENCW_D1, maxHeight=ENCH_D1});
  end
  if ENABLE_D1 == 1 then
    avcencd1 = sysctl0:avcencCreate({name="avcencd1", ws=ws, maxwidth=ENCW_D1, maxheight=ENCH_D1, outputbuffersize = 2*1024*1024});
  end
  if ENABLE_D1_LOCALSTORAGE == 1 then
    avcencd1l = sysctl0:avcencCreate({name="avcencd1l", ws=ws, maxwidth=ENCW_D1, maxheight=ENCH_D1, outputbuffersize = 2*1024*1024});
  end
end

if ENABLE_CIF == 1 or ENABLE_CIF_LOCALSTORAGE == 1 then
  -- 3rd CIF
  nvppenccif = sysctl0:nvppCreate({name="nvppcif", maxWidth=ENCW_CIF, maxHeight=ENCH_CIF});
  if ENABLE_CIF == 1 then
    avcenccif = sysctl0:avcencCreate({name="avcenccif", ws=ws, maxwidth=ENCW_CIF, maxheight=ENCH_CIF, outputbuffersize = 512*1024});
  end
  if ENABLE_CIF_LOCALSTORAGE == 1 then
    avcenccifl = sysctl0:avcencCreate({name="avcenccifl", ws=ws, maxwidth=ENCW_CIF, maxheight=ENCH_CIF, outputbuffersize = 512*1024});
  end
end

ws:showrate({enable=SHOWRATE});
ws:start();
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0:setOutput480i();
end

-- is_420 argument is not used
-- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
--Setup_VPP(lvpp0, 1280, 720, 0, 1, 2, 2)
if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 1, 4, 4)
else
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 1, 2, 2)
  lvpp0:setParam({param=Q_LVPP_CMP_PICT_WIDTH,       value=LVPP_OUTPUT_WIDTH});
  lvpp0:setParam({param=Q_LVPP_CMP_PICT_HEIGHT,      value=LVPP_OUTPUT_HEIGHT});
end
if (VIN == "sensor") then
    lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
elseif (VIN == "component") then
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=261});
elseif (VIN == "hdmi") then
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=171});
end
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=0});

if (VIN_FORMAT == "720p") then
  lvpp0:setParam({param=Q_LVPP_CMP_INPUT_DECIMATION, value=2});
end
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=VIN_STC});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
-- FIXTHIS: Need to check the aspect ratio stuff
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});

-- Don't crop any thing
lvpp0:crop({cropWidth=HSIZE, cropHeight=VSIZE, horzOffset=0, vertOffset=0});

if (VIDEO_ONLY == 0) then
  lvpp0:setParam({param=Q_LVPP_CMP_AV_SYNCH_ENABLE, value=1});
end
lvpp0:activateCfg();

-----------------------------------
-- setup the median filter
--function filterHorzMedian( vpp, threshold_luma, threshold_chroma )
  -- Non-linear impulse noise reduction filter
  -- defaults 38, 20
--filterHorzMedian( lvpp0, 38, 20 )
-- to disable
filterHorzMedian( lvpp0, 255, 255)

-- setup spatial filter
-- filterHorzEdgeAdapt(vpp, threshold1, threshold2 , lp_level )
filterHorzEdgeAdapt(lvpp0, 16, 8, 1 )
-- to disable
-- filterHorzEdgeAdapt(lvpp0, 0, 0, 4 )

-- setup temporal filter
-- filterTemporalMotion( vpp, colorEnableMask, YcorrIndex, YlowThr, YhighThr, YgainIndex0, YgainIndex1, CgainIndex0, CgainIndex1, vMotFiltBypass )
filterTemporalMotion( lvpp0, 0, 0, 32, 235, 5, 5, 5, 5, 0)
-- to disable:
--filterTemporalMotion( lvpp0, 0, 0, 32, 235, 0, 0, 0, 0, 0)
-----------------------------------

-- create and configure audio input
if (VIDEO_ONLY == 0) then
  sain0 = sysctl0:sainCreate({name="sain0",port=0});

  sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_MONO_L});
  sain0:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
  sain0:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
  sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
  sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
  sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
  sain0:activateCfg();

  -- create and bind audio encoder
-- Following is for SDK 4/4.01
--  audenc0 = sysctl0:audencCreate({name="audenc0", ws=ws, codecType=AUDIO_CODEC, id=AUDIOSTREAMID });
-- Following is for SDK 5
  audenc0 = sysctl0:audencCreate({name="audenc0", ws=ws, codecType=AUDIO_CODEC });
  audenc0:setStreamID({streamid=AUDIOSTREAMID});
  audenc0:bindAudioInput({src=sain0});

  -- configure audio encoder
  audenc0:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=Q_AUDENC_CFP_CHANNEL_MONO});
  audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=SAMPLERATE});
  audenc0:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
  audenc0:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
  --codec specific configuration
  audenc0:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q711_LAW});
  audenc0:setParam({param=Q_AUDENC_CMP_QAC_AUDIO_OBJECT_TYPE, value=QAC_AOT});
  audenc0:activateCfg();
end

if (ENABLE_HD == 1) then
  avcenc_720p25(avcenchd, PROFILE_HD, LEVEL_HD, VBITRATE_HD, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")

  avcenchd:setTransferSize({size=HD_AVCENC_XFER_SIZE});
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  if (PREVIEW == 2) then
    avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=1});
  end
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  --avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_CABAC, value=0});
  avcenchd:setVideoEncParam({param=Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, value=ENC_GUARD_BAND_PER_MB});
 
  if (ENABLE_OT == 1) then
    -- object tracking parameter settings
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                      value=1});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,           value=12});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,       value=9});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,       value=15});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,       value=256});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,       value=256});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,           value=9});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                  value=30});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                  value=30});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,        value=0});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,        value=0});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,value=0});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,            value=5});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,     value=300});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,      value=255});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,              value=30});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,       value=2});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,   value=1});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,          value=1});
    avcenchd:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,        value=1});
    -- end object tracking parameter settings

  end
  
  avcenchd:activateVideoEncCfg();

  avcenchd:setStreamID({streamid=videoStreamId_hd});
end

if (ENABLE_HD_LOCALSTORAGE == 1) then
    avcenc_720p25(avcenchdl, PROFILE_HD, LEVEL_HD, VBITRATE_HD_LOCALSTORAGE, gopstruct_hd_localstorage, "frame")

  avcenchdl:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenchdl:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
  avcenchdl:activateVideoEncCfg();

  avcenchdl:setStreamID({streamid=videoStreamId_hd_localstorage});
  avcenchdl:bindVideoInput({src=lvpp0});
end

if (ENABLE_HD_LOCALSTORAGE == 1) then
  if (OUTFILE_FORMAT == "h4v") then
    ws:open({type="h4v", file=OUTFILENAME_HD, sid = {videoStreamId_hd_localstorage}});
  else
    ws:open({type="file", file=OUTFILENAME_HD, sid = {videoStreamId_hd_localstorage, AUDIOSTREAMID}});
  end

end

if ((PREVIEW == 1) or (PREVIEW == 2)) then
  if (VIDEO_ONLY == 0) then
	-- Echo audio to output
	avsn0:bindAudioInput({src=sain0});
	ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO});
	avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
        avoc0:enableAudioChannel({ch=ach0, enable=1});
	avoc0:activateAudioChannelCfg();
  end
end

if (PREVIEW == 1) then
	avsn0:bindVideoInput({src=lvpp0});
elseif (PREVIEW == 2) then
	avsn0:bindVideoInput({src=avcenchd});
end
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  vch0=avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
  avoc0:activateVideoChannelCfg();
  if (ENABLE_OT == 1) then
    osdinit({ avoc=avoc0,
        width=OSD_DISPLAY_WIDTH,
        height=OSD_DISPLAY_HEIGHT,
        x=OSD_X_OFFSET,y=OSD_Y_OFFSET,
        memory=OSD_MEMORY_SIZE});

    --fbset
    os.execute("fbset 720x480 LUT8");  -- LUT8 format
    os.execute("fbset enable 1");
    os.execute("fbset ck_mode 1");
    os.execute("fbset alpha_select 0");
    os.execute(string.format("fbset hzoom %d", ZOOM))
    os.execute(string.format("fbset vzoom %d", ZOOM))

  end
end

if ENABLE_D1 == 1 or ENABLE_D1_LOCALSTORAGE == 1 then
-- for 2nd D1 stream
  -- NVPP instance is used only if HD encoding is also needed
  if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
    Setup_VPP(nvppencd1, HSIZE, VSIZE, 0, 1, 2, 2)
    nvppencd1:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_D1});
    nvppencd1:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_D1});
    nvppencd1:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
    nvppencd1:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
    nvppencd1:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
    nvppencd1:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
    nvppencd1:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=VIN_OUTPUT_STC});
    nvppencd1:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
    nvppencd1:activateCfg();
    nvppencd1:bindInput({src=lvpp0});
  end
  if ENABLE_D1 == 1 then
    print("*** using high quality D1")
    avcenc_480i_highquality(avcencd1, PROFILE_D1, LEVEL_D1, VBITRATE_D1, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
    avcencd1:setTransferSize({size=D1_AVCENC_XFER_SIZE});

    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
    avcencd1:setVideoEncParam({param=Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, value=ENC_GUARD_BAND_PER_MB});

    if (ENABLE_OT == 1) and (ENABLE_HD == 0) then
      -- object tracking parameter settings
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                      value=1});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,           value=12});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,       value=9});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,       value=15});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,       value=256});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,       value=256});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,           value=9});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                  value=30});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                  value=30});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,        value=0});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,        value=0});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,value=0});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,            value=5});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,     value=300});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,      value=255});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,              value=30});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,       value=2});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,   value=1});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,          value=1});
      avcencd1:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,        value=1});
      -- end object tracking parameter settings
    end
    avcencd1:activateVideoEncCfg();
    if(ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
      avcencd1:bindVideoInput({src=nvppencd1});
    end

    avcencd1:setStreamID({streamid=videoStreamId_d1});
    avcencd1:setAckInterval({interval=8});
 end


  if ENABLE_D1_LOCALSTORAGE == 1 then
    print("*** using high quality D1")
    avcenc_480i_highquality(avcencd1l, PROFILE_D1, LEVEL_D1, VBITRATE_D1_LOCALSTORAGE, gopstruct_d1_localstorage, "frame")
    avcencd1l:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
    avcencd1l:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});

    avcencd1l:activateVideoEncCfg();
    if(ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
      avcencd1l:bindVideoInput({src=nvppencd1});
    else
      avcencd1l:bindVideoInput({src=lvpp0});
    end
    if (OUTFILE_FORMAT == "h4v") then
      ws:open({type="h4v", file=OUTFILENAME_D1, sid = {videoStreamId_d1_localstorage}});
    else
      ws:open({type="file", file=OUTFILENAME_D1, sid = {videoStreamId_d1_localstorage, AUDIOSTREAMID}});
    end


    avcencd1l:setStreamID({streamid=videoStreamId_d1_localstorage});
    avcencd1l:setAckInterval({interval=8});
  end
end

if ENABLE_CIF == 1 or ENABLE_CIF_LOCALSTORAGE == 1 then
-- for 3rd CIF stream
  Setup_VPP(nvppenccif, LVPP_OUTPUT_WIDTH, LVPP_OUTPUT_HEIGHT, 0, 1, 2, 2)
  nvppenccif:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_CIF});
  nvppenccif:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_CIF});
  nvppenccif:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
  nvppenccif:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
  nvppenccif:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
  nvppenccif:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
  nvppenccif:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=VIN_OUTPUT_STC});
  nvppenccif:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
  nvppenccif:activateCfg();
  nvppenccif:bindInput({src=lvpp0});

  if ENABLE_CIF == 1 then
    avcenc_480i_highquality(avcenccif, PROFILE_CIF, LEVEL_CIF, VBITRATE_CIF, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "frame")
    avcenccif:setTransferSize({size=CIF_AVCENC_XFER_SIZE});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE, value=1});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
    avcenccif:setVideoEncParam({param=Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, value=ENC_GUARD_BAND_PER_MB});

    avcenccif:activateVideoEncCfg();
    avcenccif:bindVideoInput({src=nvppenccif});

    avcenccif:setStreamID({streamid=videoStreamId_cif});
    avcenccif:setAckInterval({interval=8});
  end
  if ENABLE_CIF_LOCALSTORAGE == 1 then
    avcenc_480i_highquality(avcenccifl, PROFILE_CIF, LEVEL_CIF, VBITRATE_CIF_LOCALSTORAGE, gopstruct_cif_localstorage, "frame")
    avcenccifl:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
    avcenccifl:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});

    avcenccifl:activateVideoEncCfg();
    avcenccifl:bindVideoInput({src=nvppenccif});
    if (OUTFILE_FORMAT == "h4v") then
      ws:open({type="h4v", file=OUTFILENAME_CIF, sid = {videoStreamId_cif_localstorage}});
    else
      ws:open({type="file", file=OUTFILENAME_CIF, sid = {videoStreamId_cif_localstorage, AUDIOSTREAMID}});
    end


    avcenccifl:setStreamID({streamid=videoStreamId_cif_localstorage});
    avcenccifl:setAckInterval({interval=8});
  end

end

CreateSnapShotObj();
CreateMJPEGObj();
SetupSnapShotObj();
SetupMJPEGObj();
LinkSnapShotObj();
LinkMJPEGObj();


-- start output
if ((PREVIEW == 1) or (PREVIEW == 2)) then
  avoc0:start();
  avsn0:start();
end

-- start encoders
if (VIDEO_ONLY == 0) then
  audenc0:record();
end


if(ENABLE_HD == 1) then
  recordhd();
end

if(ENABLE_HD_LOCALSTORAGE == 1) then
  avcenchdl:record();
end

if (ENABLE_JPEGSNAPSHOT == 1) then
    jpegenc0:record();
    if(ENCH_JPEGSNAPSHOT ~= LVPP_OUTPUT_HEIGHT) or (ENCW_JPEGSNAPSHOT ~= LVPP_OUTPUT_WIDTH) then
      nvpp_jpegsnapshot:start();
    end
end

if (ENABLE_MJPEG == 1) or (ENABLE_MJPEG_LOCALSTORAGE == 1) then
  jpegenc1:record();
  if(ENCH_MJPEG ~= LVPP_OUTPUT_HEIGHT) or (ENCW_MJPEG ~= LVPP_OUTPUT_WIDTH) then
    nvpp_mjpeg:start();
  end
end

if ENABLE_D1 == 1 then
  recordd1();
  if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
    nvppencd1:start();
  end
end
if ENABLE_CIF == 1 then
  recordcif();
end

if ENABLE_D1_LOCALSTORAGE == 1 then
  avcencd1l:record();
  if ENABLE_D1 == 0 then
    if (ENABLE_HD == 1) or (ENABLE_HD_LOCALSTORAGE == 1) then
      nvppencd1:start();
    end
  end
end

if ENABLE_CIF_LOCALSTORAGE == 1 then
  avcenccifl:record();
  if ENABLE_CIF == 0 then
    nvppenccif:start();
  end
end

if (VIDEO_ONLY == 0) then
  sain0:start();
end

lvpp0:start();

if (LED == 1) then
  -- Turn on LED D1 to show that the system has started
  os.execute( "echo 0 >/proc/driver/gpio/native/64" )
end

function mycrop(width,height,intX,intY)
    cropd1(width, height, intX, intY);
    end

function uncrop()
    cropd1(1280, 720, 0, 0);
end

